#include "addlanguagedialog.h"
#include "ui_addlanguagedialog.h"
#include <QTranslator>
#include <QToolTip>
#include <QCompleter>

const QList<QPair<QString, QPair<QString, QString>>> AddLanguageDialog::supportedLanguageList {
        {"zh_CN", {"简体中文", QObject::tr("简体中文")}},
        {"en_US", {"English", QObject::tr("English")}},
        {"bo_CN", {"བོད་ཡིག",   QObject::tr("བོད་ཡིག")}}
    };

AddLanguageDialog::AddLanguageDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AddLanguageDialog),
    mSearchLineEdit(new KSearchLineEdit(this))
{
    ui->setupUi(this);
    this->setWindowTitle(tr("Add Language"));
    mSearchLineEdit->setPlaceholderText(tr("Search"));
    ui->frame->layout()->addWidget(mSearchLineEdit);
    ui->tableView->setStyleSheet("QTableView{border: 1px solid palette(midlight);}"
                                  "QTableView::item{padding-left: 12px;border:0;}");
    mode = new QStandardItemModel(this);
    ui->tableView->setModel(mode);
    ui->tableView->setAlternatingRowColors(true);
    ui->tableView->setMouseTracking(true);
    ui->tableView->setSelectionMode(QAbstractItemView::SelectionMode::SingleSelection);
    ui->tableView->setSelectionBehavior(QAbstractItemView::SelectionBehavior::SelectRows);
    connect(ui->tableView, &QTableView::entered,
            this, [=](const QModelIndex &index){
        QString mStr = index.data().toString();
        QFontMetrics fontMetrics(this->font());
        int fontSize = fontMetrics.width(mStr);
        if (fontSize - 16 > 416) {
            QToolTip::showText(QCursor::pos(), mStr);
        }
    });
    ui->tableView->verticalHeader()->setVisible(false);//隐藏垂直表头
    ui->tableView->horizontalHeader()->setVisible(false);//隐藏水平表头
    ui->tableView->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ui->tableView->setShowGrid(false);//不显示表格线
    ui->tableView->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);//水平表格自动拉伸
    ui->tableView->verticalHeader()->setDefaultSectionSize(36);//行高默认为36
    this->setWindowModality(Qt::ApplicationModal); //模态

    ui->tableView->setEditTriggers(QAbstractItemView::NoEditTriggers);//不可编辑
    mode->setColumnCount(1);

    /*暂时使用自定义的语言，不使用apt获取的
    languageInfo = new LanguageInfo;
    connect(languageInfo,
            &LanguageInfo::initFinished,
            this,
            [=](QMap<QString, QString> availableLangMap,QMap<QString, QString> installedLangMap){
            this->availableLangMap = availableLangMap;
            this->installedLangMap = installedLangMap;
            mode->setRowCount(availableLangMap.count());

            QMap<QString, QString>::const_iterator it = availableLangMap.constBegin();
            for (int i = 0; i < mode->rowCount(); i++, it++) {
                mode->setItem(i, new QStandardItem(it.value()));
            }
            initCompleter();
    });
    languageInfo->init();
    */
    mode->setRowCount(supportedLanguageList.count());
    for (int i = 0; i < supportedLanguageList.count(); ++i) {
        mode->setItem(i, new QStandardItem(SHOW_LANGUAGE_NAME(supportedLanguageList.at(i).second.first,
                                                              supportedLanguageList.at(i).second.second)));
    }
    initCompleter();
    connect(ui->yesBtn,&QPushButton::clicked,this,[=]() {
        QModelIndexList indexList =  ui->tableView->selectionModel()->selectedRows();
        QStringList selectedLanguageList = {};
        for (QModelIndex a:indexList) {
            selectedLanguageList.append(ui->tableView->model()->data(a).toString());
        }
        Q_EMIT addShowLanguage(selectedLanguageList);
        this->hide();
    });
    connect(ui->noBtn,&QPushButton::clicked,this,[=]() {
        this->hide();
    });
}

AddLanguageDialog::~AddLanguageDialog()
{
    delete ui;
}

void AddLanguageDialog::initCompleter()
{
    QCompleter *m_completer = new QCompleter(this);
    m_completer->setModel(mode);
    m_completer->setMaxVisibleItems(6);
    m_completer->setFilterMode(Qt::MatchContains);
    m_completer->setCaseSensitivity(Qt::CaseInsensitive);
    if (mSearchLineEdit) {
        mSearchLineEdit->setCompleter(m_completer);
    }

    connect(mSearchLineEdit, &QLineEdit::returnPressed, this, [=]() {
        if (!mSearchLineEdit->text().isEmpty()) {
            QList<QStandardItem*> itemList = mode->findItems(mSearchLineEdit->text());
            if (!itemList.isEmpty()) {
                QModelIndex m_modeIndex = mode->indexFromItem(itemList.first());
                ui->tableView->scrollTo(m_modeIndex);
                int index = m_modeIndex.row();
                ui->tableView->selectRow(index);
            }
        }
    });

#if QT_VERSION <= QT_VERSION_CHECK(5, 12, 0)
    connect(m_completer, static_cast<void(QCompleter::*)(const QString &)>(&QCompleter::activated),
            [=](const QString &text) {
#else
    connect(m_completer, QOverload<const QString &>::of(&QCompleter::activated),
            [=](const QString &text) {
#endif
        Q_UNUSED(text);
        Q_EMIT mSearchLineEdit->returnPressed();
    });
}

void AddLanguageDialog::showEvent(QShowEvent *e)
{
    mSearchLineEdit->clear();
    ui->tableView->clearSelection();
    QDialog::showEvent(e);
}
